import asyncio
import socketio
from serve import rpc, exported, rpc_serve
from stub import get_service_proxy

@rpc()
class ClientSideService:
    @exported
    def mul(self, a, b):
        print('mul', a, b)
        return a * b

async def test_func(sio, service):
    print('run test_func')
    print(await service.add(1, 2))
    print(await service.addAsync(1, 2))

    try:
        await service.throwError()
    except Exception as e:
        print(e)

    try:
        await service.sub(1, 2)
    except Exception as e:
        print(e)

    await service.callClientMul()
    print('done !!!')
    await sio.disconnect()


async def main():
    sio = socketio.AsyncClient()
    print('connecting')
    await sio.connect('ws://localhost:3333')
    service = get_service_proxy(sio, 'ServerSideService')
    rpc_serve(sio, ClientSideService())
    await asyncio.gather(
        sio.wait(),
        test_func(sio, service)
    )

asyncio.run(main())



import functools
import inspect
from asyncio import Future, get_running_loop

__all__ = ['RpcException', 'get_service_proxy']

class RpcException(Exception):
    pass

class RpcProxy:
    def __init__(self, transporter, service_name, loop=None):
        self.loop = loop if loop else get_running_loop()
        self.transporter = transporter
        self.ns = service_name
        self.seq = 1
        self.futures = {}

    def _on_rpc_return(self, seq, value, error):
        future = self.futures.get(seq)
        if not future:
            return

        if error:
            future.set_exception(RpcException(error))
        else:
            future.set_result(value)

        del self.futures[seq]

    async def __rpc_call(self, method, *params):
        em = self.transporter.emit('__rpc_call__', {
            'ns': self.ns,
            'seq': self.seq,
            'method': method,
            'params': params
        })
        if inspect.isawaitable(em):
            await em

        future = self.loop.create_future()
        self.futures[self.seq] = future
        self.seq = self.seq + 1
        return await future

    async def __rpc_call_no_ret(self, method, callback, *params):
        em = self.transporter.emit('__rpc_call__', {
            'ns': self.ns,
            'seq': 0,
            'method': method,
            'params': params
        })
        if inspect.isawaitable(em):
            await em

    def __getattr__(self, method):
        func = functools.partial(self.__rpc_call, method)
        func.noret = functools.partial(self.__rpc_call_no_ret, method)
        setattr(self, method, func)
        return func

def ensure_rpc_proxy(transporter):
    proxies = getattr(transporter, '__rpc_proxies', None)
    if proxies:
        return proxies
    proxies = {}
    transporter.__rpc_proxies = proxies

    def on_rpc_return(data):
        proxy = proxies.get(data['ns'])
        if not proxy:
            return
        proxy._on_rpc_return(data['seq'], data.get('value'), data.get('error'))

    transporter.on('__rpc_return__', on_rpc_return)
    return proxies

def get_service_proxy(transporter, service_name, event_loop=None):
    proxies = ensure_rpc_proxy(transporter)
    proxy = proxies.get(service_name)
    if proxy:
        return proxy
    proxy = RpcProxy(transporter, service_name, event_loop)
    proxies[service_name] = proxy
    return proxy



import inspect

__all__ = ['rpc', 'exported', 'rpc_serve', 'rpc_stop']

def rpc(name=None):
    def rpc_decorator(klass):
        klass.__rpc_options = {
            'name': name if name else klass.__name__,
        }
        return klass
    return rpc_decorator

def exported(method):
    method.__rpc_export = True
    return method

def ensure_rpc_services(transporter):
    services = getattr(transporter, '__rpc_services', None)
    if services:
        return services

    services = {}
    transporter.__rpc_services = services

    async def on_rpc_call(data):
        service = services.get(data['ns'])
        if not service:
            em = transporter.emit('__rpc_return__', {
                'ns': data['ns'],
                'seq': data['seq'],
                'error': f'Service {data["ns"]} not found'
            })
            if inspect.isawaitable(em):
                await em
        method = getattr(service, data['method'], None)
        if not method or not getattr(method, '__rpc_export', False):
            em = transporter.emit('__rpc_return__', {
                'ns': data['ns'],
                'seq': data['seq'],
                'error': f'Rpc method {data["method"]} not found'
            })
            if inspect.isawaitable(em):
                await em

        try:
            value = method(*data['params'])
            if inspect.isawaitable(value):
                value = await value
            em = transporter.emit('__rpc_return__', {
                'ns': data['ns'],
                'seq': data['seq'],
                'value': value
            })
            if inspect.isawaitable(em):
                await em
        except Exception as e:
            em = transporter.emit('__rpc_return__', {
                'ns': data['ns'],
                'seq': data['seq'],
                'error': str(e)
            })
            if inspect.isawaitable(em):
                await em

    transporter.on('__rpc_call__', on_rpc_call)
    return services

def rpc_serve(transporter, service):
    options = getattr(service, '__rpc_options', None)
    if not options:
        raise Exception('Not a valid rpc service')
    services = ensure_rpc_services(transporter)
    services[options['name']] = service

def rpc_stop(transporter, service_name):
    services = getattr(transporter, '__rpc_services', None)
    if not services:
        return
    if service_name in services:
        del services[service_name]



from setuptools import setup

setup(
    name='anyrpc',
    version='0.0.1',
    author='fuchen',
    author_email='ink.pad@163.com',
    url='https://github.com/fuchen/any-rpc-py',
    description=u'Python implementation of any-rpc',
    packages=['anyrpc'],
    install_requires=[],
    entry_points={
    }
)
